"""
Script to import Nike fuel data from a CSV format like the following:

distance,miles,steps,pace,fuel,start_time,device,calories,duration
2.70,2.69928887137,5517,(33'46/mi),2714,2013-06-08T05:00:00Z,FUELBAND,839,9:37:00

The result will be a new Day One entry for each line in the CSV file.
"""

import argparse
import collections
from datetime import datetime
import csv
import os
import re
import uuid

DAYONE_ENTRIES = '/Users/durden/Dropbox/Apps/Day One/Journal.dayone/entries/'

# This text will be inserted into the first line of all entries created, set to
# '' to remove this completely.
HEADER_FOR_DAYONE_ENTRIES = 'Nike Fuel'

# Note the strange lack of indentation on the {entry_text} b/c day one will
# display special formatting to text that is indented, which we want to avoid.
ENTRY_TEMPLATE = """
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
<dict>
    <key>Creation Date</key>
    <date>{start_time}</date>
    <key>Entry Text</key>
    <string> {entry_title}
- Fuel: {fuel} points
- Steps: {steps}
- Distance: {distance} miles
- Calories: {calories}
- Device: {device}

#nikefuel #fitness
    </string>
    <key>Starred</key>
    <false/>
    <key>Tags</key>
    <array>
        <string>nikefuel</string>
        <string>fitness</string>
    </array>
    <key>UUID</key>
    <string>{uuid_str}</string>
</dict>
</plist>
"""


def _parse_args():
    """Parse sys.argv arguments"""

    parser = argparse.ArgumentParser(
                                description='Export NikeFuel data to Day One')

    parser.add_argument('-f', '--file', action='store',
                        dest='input_file', required=True,
                        help='CSV file to import from')

    parser.add_argument('-v', '--verbose', default=False, action='store_true',
                        dest='verbose', required=False,
                        help='Verbose debugging information')

    parser.add_argument('-t', '--test', default=False, action='store_true',
                        dest='test', required=False,
                        help=('Test import by creating Day one files in local '
                              'directory for inspect'))

    def _datetime(str_):
        """Convert date string in YYYY-MM-DD format to datetime object"""

        if not str_:
            return None

        try:
            date = datetime.strptime(str_, '%Y-%m-%d')
        except ValueError:
            msg = 'Invalid date format, should be YYYY-MM-DD'
            raise argparse.ArgumentTypeError(msg)

        return date

    parser.add_argument('-s', '--since', type=_datetime,
                        help=('Only process entries starting with YYYY-MM-DD '
                              'and newer'))

    return vars(parser.parse_args())


def _create_nikeplus_entry(activity, directory, verbose):
    """
    Create/write day one file with given nike plus activity

    activity should be a named tuple
    """

    # Create unique uuid without any specific machine information
    # (uuid() vs.  uuid()) and strip any '-' characters to be
    # consistent with dayone format.
    uuid_str = re.sub('-', '', str(uuid.uuid4()))

    file_name = '%s.doentry' % (uuid_str)
    full_file_name = os.path.join(directory, file_name)

    activity_dict = activity._asdict()
    activity_dict['uuid_str'] = uuid_str

    with open(full_file_name, 'w') as file_obj:
        text = ENTRY_TEMPLATE.format(entry_title=HEADER_FOR_DAYONE_ENTRIES,
                                     **activity_dict)
        file_obj.write(text)

    if verbose:
        print 'Created entry for %s: %s' % (activity.start_time, file_name)


def read_entries(filename, start_date=None):
    """
    Read and yield namedtuple for entries from filename

    if start_date is given as a datetime object only entries that happened on
    or after that date will be returned.
    """

    with open(filename, 'r') as file_obj:
        csv_reader = csv.reader(file_obj)
        header = csv_reader.next()

        activity = collections.namedtuple('activity', header)

        for row in csv_reader:
            # Create named tuple then use it's API to convert to a dict we can
            # easily expand to format the entry text without hardcoding any
            # names in the code itself.  We are heavily dependent on the names
            # in the entry template matching the header line though.
            entry = activity(*row)

            # Date information in nikeplus is separated from time information
            # with a 'T'
            date = entry.start_time.split('T')[0].strip()
            entry_date = datetime.strptime(date, '%Y-%m-%d')

            if start_date is None or entry_date >= start_date:
                yield entry


def main():
    args = _parse_args()

    if args['test']:
        directory = './test'
        try:
            os.mkdir(directory)
        except OSError as err:
            print 'Warning: %s' % (err)
    else:
        directory = DAYONE_ENTRIES

    for entry in read_entries(args['input_file'], args['since']):
        _create_nikeplus_entry(entry, directory, args['verbose'])


if __name__ == '__main__':
    main()



#filename = '/Users/durden/Dropbox/twitter_backups/tweets/data/js/tweets/2013_06.js'

from datetime import datetime
import json
import re

# This doesn't work on Python built-in for OS X, 'z' is not valid directive
# even though it's specified in docs??
# Looks like datetime.strptime only supports what time.strftime supports and is
# platform dependent.  So, apparently this isn't supported on the Python
# version that comes with OS X (b/c apple is silly or b/c apple's C runtime
# doesn't support it under the hood).

# As a workaround, could just parse the string's first 4 tokens and last token
# with split() and pass those to datetime.strptime b/c that will strip off
# the timezone.
#datetime.strptime('%a %b %d %H:%M:%S %z %Y', d)


def tweet_to_markdown(tweet):
    text = tweet['text']
    urls = tweet['entities']['urls'] or []

    # Replace all urls with markdown version
    for url in urls:
        text = re.sub(r'(%s)' % (url['url']),
                      r'[%s](%s)' % (url['display_url'], url['expanded_url']),
                      text)
    return text


def main(filename):
    tweets = open(filename, 'r')

    # Discard assignment line
    tweets.readline()

    json_tweets = tweets.read()

    for tweet in json.loads(json_tweets):
        # FIXME: Need to detect date from tweet json:
        #   - Create new dayone file for each day
        print tweet_to_markdown(tweet)


if __name__ == '__main__':
    import sys
    main(sys.argv[1])



#!/usr/bin/python
# -*- mode: python;  coding: utf-8; -*-
"""
This service will extract data from the sqlite database of the pedometer++
iOS app (http://crossforward.com) and exports them as Day One Journal
(http://dayoneapp.com/) entries.

The database is used from the iTunes backup of the iOS device which you can locate
from within the iTunes Preferences->Devices via the context menu (Reveal in Finder).

The database file name is assumed to be 'dda8ace3c3f41792dd620ef4269a1344031686a7'

Also, it's encouraged to run this with the '-o test' option first so that all your
Day One entires will be created in a local directory called 'temp.'  This will
allow you to inspect the conversion.  You can manually copy a few select
entries into your Day One 'entries/' folder to ensure you approve of the
formatting and can easily make any formatting adjustments.  Then, you can run
this module again to fully import entries into Day One.

Get help with:

./dayonetools/main.py pedometerpp -h

Example with a fictious device and output collected in test folder:

./dayonetools/main.py pedometerpp -d 0123456789ABCDEFEDCBBCDEFEDCBA9876543210 -o test

"""
from __future__ import unicode_literals

__author__ = 'jotefa'

import argparse
import shutil
from dayonetools.services import get_outfolder_names
import os
import sqlite3 as sqlite
import datetime
import pytz
import plistlib
import uuid

SERVICENAME = 'pedometerpp'
SERVICEVERSION = '1.0a2'
SERVICEID = 'de.jotefa.d1tools.pedometerpp'


class PedometerPP():

    def __init__(self):
        parser = argparse.ArgumentParser(
            description='Import Pedometer++ data from your iPhone Backup into your Day One Journal'
        )
        parser.add_argument(
            '-d', '--device', action='store',
            dest='device', required=True,
            help='Backup folder of the iOS device'
        )
        parser.add_argument(
            '-t', '--timezone', default='Europe/Berlin', action='store',
            dest='timezone', required=False,
            help="The English name of the timezone of the pedometer (iPhone),"
                 " e.g. 'America/Chicago' or 'Africa/Johannesburg', default: 'Europe/Berlin'"
        )
        parser.add_argument(
            '-o', '--out', default='auto', action='store',
            dest='outfolder', required=True,
            help="Day One Journal folder or 'auto' or 'test'"
        )
        parser.add_argument(
            '-v', '--verbose', default=False, action='store_true',
            dest='verbose', required=False,
            help='Verbose debugging information'
        )
        self.args = vars(parser.parse_args())

        # FIXME: Add progress output for --verbose
        print self.args, file

        # Prepare output folders
        self.d1folder, self.tempfolder = get_outfolder_names(
            SERVICENAME,
            self.args['outfolder'],
            self.args['verbose']
        )
        print '\nExporting to “{0}”'.format(self.d1folder)
        print '\nTemporary files saved in “{0}”'.format(self.tempfolder)

        # Prepare input database
        backupfolder = '~/Library/Application Support/MobileSync/Backup'
        # TODO: Add testing for OS when Day One becomes available on Windows or other OS

        dbbackup = os.path.join(
            os.path.expanduser(backupfolder),
            self.args['device'],
            'dda8ace3c3f41792dd620ef4269a1344031686a7'  # Name of the pedometer++ database backup
        )
        print '\nCollecting data from “{0}”'.format(dbbackup)

        # FIXME: Check existence

        # Copy the database into tempFolder to avoid messing up the “official”
        # backup with temp files generated during sqlite access
        # It’s a waste, but only 30~KB per half a year
        self.dbfile = os.path.join(self.tempfolder, 'pedometerpp.sqlite')
        shutil.copy(dbbackup, self.dbfile)

        # Initialize our data collection
        self.entries = {}

    def collect_entries(self):
        """
        Load the database entries into a python dict
        Key is the UTC time of an entry
        """
        con = sqlite.connect(self.dbfile)
        cur = con.cursor()

        cur.execute("SELECT * FROM ZSTEPCOUNT ORDER BY ZTIMESTAMP")
        col_names = [cn[0] for cn in cur.description]
        print '\n', col_names

        for row in cur.fetchall():
            #print row
            # Pedometer++ saves a ZDATESTRING which is a mess of localized names
            # and US sequence. We ignore it and interpret the ZTIMESTAMP, which
            # is shifted by 31 years.
            dt = datetime.datetime.fromtimestamp(row[4])
            # We set the time to 23:59 to make it the last item of a Day One day
            dt = dt.replace(year=dt.year+31, hour=23, minute=59)
            # Convert from the device time zone to UTC as expected by Day One
            dt = pytz.timezone(self.args['timezone']).localize(dt)
            dt = dt.astimezone(pytz.utc)
            #print dt.tzinfo, dt.dst()

            print '{1} • {0[0]:4} {0[1]:5} {0[2]:5} {0[3]:10} {0[4]:10} {0[5]}'.format(row, dt)
            self.entries[dt] = {
                'ent': row[1],  # FIXME: I have no idea what that is
                'opt': row[2],  # FIXME: I have no idea what that is
                'steps': row[3],
            }

    def export_d1(self):
        """
        Loop through entries by date and count weekly and monthly sums
        Write a plist for every entry into the output folder
        """
        sumweek = 0
        summonth = 0

        # FIXME: Make text localizable

        def created1entry(edate, ekind, esteps, etext):
            entry = {
                'Creator': {
                    'Software Agent': '{0} {1}'.format(SERVICEID, SERVICEVERSION),
                    'Generation Date': datetime.datetime.utcnow(),
                },
                'UUID': str.upper(uuid.uuid4().hex),
                'Creation Date': edate,
                'Time Zone': self.args['timezone'],
                'Tags': [SERVICENAME, 'M7Steps', '⚗Auto', ekind],
                'Starred': False,
                'Step Count': esteps,
                'Entry Text': etext,
            }
            #plistlib.writePlist(entry, os.path.join(outfolder, entry['UUID']+'.doentry') )

            # Day One names files with the uuid used in the file but other names seem to work as well
            # So we use the date and the service name to create a name
            # FIXME: Test for existing entry file
            plistlib.writePlist(
                entry,
                os.path.join(self.d1folder, '{0}_{1}.doentry'.format(
                    edate.strftime('%Y-%m-%dT%H-%M-%SZ'),
                    SERVICENAME
                ))
            )

        for i in sorted(self.entries):
            # Create entry for the last month on the first of the current month
            if i.day == 1:
                ii = i.replace(second=40) - datetime.timedelta(days=1)
                print('{0} {1:10} {2:4} {3:10} ∑ {4}'.format(
                    ii, 0,
                    self.entries[i]['opt'],
                    summonth,
                    ii.strftime('%B %Y')
                ))
                created1entry(
                    ii, '∑ Month', summonth,
                    'Schritte im Monat {1}: {0}'.format(summonth, ii.strftime('%Y-%b'))
                )
                summonth = 0
            summonth += self.entries[i]['steps']

            # Create entry for this Day
            print('{0} {1:10} {2:4}'.format(i, self.entries[i]['steps'], self.entries[i]['opt']))
            created1entry(
                i, '∑ Day', self.entries[i]['steps'],
                'Schritte heute: {0}'.format(self.entries[i]['steps'])
            )

            # We use ISO weeks, so we create a summary on Sunday
            sumweek += self.entries[i]['steps']
            (isoYear, isoWeek, isoWeekday) = i.isocalendar()
            if isoWeekday == 7:  # sunday
                ii = i.replace(second=20)
                print('{0} {1:10} {2:4} {3:10} ∑ KW {4} {5}'.format(
                    ii, 0,
                    self.entries[i]['opt'],
                    sumweek, isoWeek, isoYear))
                created1entry(
                    ii, '∑ Week', sumweek,
                    'Schritte in Woche {1:02}-{2}: {0}'.format(sumweek, isoWeek, isoYear)
                )
                sumweek = 0


def main():
    ppp = PedometerPP()
    ppp.collect_entries()
    ppp.export_d1()

if __name__ == '__main__':
    main()



"""
This module provides a way to import data from the Habit List iPhone
application (http://habitlist.com/) into Day One Journal
(http://dayoneapp.com/) entries.

To use this module you must first do a manual export of your data from Habit
list.  This can be done by the following:
    - Open Habit List iPhone app
    - Click the 'gear' icon for settings at the bottom of the main 'Today' view
    - Choose the 'Export Data' option
    - E-mail the data to yourself
    - Copy and paste the e-mail contents into a file of your choosing
        - Remove the 'sent from iPhone' line at the end of your e-mail.  This
          will cause the script to NOT process the JSON data.
        - DO NOT REMOVE THE LAST TWO EMPTY LINES OF THE E-MAIL. WE CURRENTLY
          HAVE A BUG THAT EXPECTS THESE LINES.
        - You can choose to optionally remove the first few lines of the e-mail
          that are not JSON data, everything up to the first '[' character.
        - Again, this is optional because this module will attempt to ignore
          any non-JSON data at the START of a file.

At this point, you are ready to do the actual conversion from JSON to Day One
entires.  So, you should check all the 'settings' in this module for things you
would like to change:
    - HEADER_FOR_DAY_ONE_ENTRIES
    - DAYONE_ENTRIES
    - ENTRY_TEMPLATE
    - TIMEZONE
        - Make sure to choose the timezone of your iPhone because the Habit
          List app stores all timezones in UTC and you'll want to convert this
          to the timezone your iPhone used at the time you completed the habit.
          This will ensure your Day One entries match the time you completed
          the task and also prevent a habit from showing up more than once per
          day which can happen with UTC time if you complete a habit late in
          one day and early in the next, etc.
        - You can find a list of available timezone strings here:
            - http://en.wikipedia.org/wiki/List_of_tz_database_time_zones

Next, you can run this module with your exported JSON data as an argument like
so:
    - python services/habit_list.py -f habit_list_data.json -t

Also, it's encouraged to run this with the '-t' option first so that all your
Day One entires will be created in a local directory called 'test.'  This will
allow you to inspect the conversion.  You can manually copy a few select
entries into your Day One 'entries/' folder to ensure you approve of the
formatting and can easily make any formatting adjustments.  Then, you can run
this module again without the '-t' to fully import Habit List entries into Day
One.
"""

import argparse
import collections
from datetime import datetime
import json
import os
import re
import uuid

from dateutil import tz

from dayonetools.services import convert_to_dayone_date_string

DAYONE_ENTRIES = '/Users/durden/Dropbox/Apps/Day One/Journal.dayone/entries/'

# This text will be inserted into the first line of all entries created, set to
# '' to remove this completely.
HEADER_FOR_DAYONE_ENTRIES = 'Habit List entry'

# Note the strange lack of indentation on the {entry_text} b/c day one will
# display special formatting to text that is indented, which we want to avoid.
ENTRY_TEMPLATE = """
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
<dict>
    <key>Creation Date</key>
    <date>{date}</date>
    <key>Entry Text</key>
    <string> {entry_title}
<![CDATA[
{habits}]]>
#habits #habit_list
    </string>
    <key>Starred</key>
    <false/>
    <key>Tags</key>
    <array>
        <string>habits</string>
        <string>habit_list</string>
    </array>
    <key>UUID</key>
    <string>{uuid_str}</string>
</dict>
</plist>
"""

TIMEZONE = 'America/Chicago'


def _parse_args():
    """Parse sys.argv arguments"""

    parser = argparse.ArgumentParser(
                               description='Export Habit List data to Day One')

    parser.add_argument('-f', '--file', action='store',
                        dest='input_file', required=True,
                        help='JSON file to import from')

    parser.add_argument('-v', '--verbose', default=False, action='store_true',
                        dest='verbose', required=False,
                        help='Verbose debugging information')

    parser.add_argument('-t', '--test', default=False, action='store_true',
                        dest='test', required=False,
                        help=('Test import by creating Day one files in local '
                             'directory for inspect'))

    def _datetime(str_):
        """Convert date string in YYYY-MM-DD format to datetime object"""

        if not str_:
            return None

        try:
            date = datetime.strptime(str_, '%Y-%m-%d')
        except ValueError:
            msg = 'Invalid date format, should be YYYY-MM-DD'
            raise argparse.ArgumentTypeError(msg)

        return date.replace(tzinfo=_user_time_zone())

    parser.add_argument('-s', '--since', type=_datetime,
                        help=('Only process entries starting with YYYY-MM-DD '
                              'and newer'))

    return vars(parser.parse_args())


def _user_time_zone():
    """Get default timezone for user"""

    try:
        return tz.gettz(TIMEZONE)
    except Exception as err:
        print 'Failed getting timezone, check your TIMEZONE variable'
        raise


def _user_time_zone_date(dt, user_time_zone, utc_time_zone):
    """
    Convert given datetime string into a yyyy-mm-dd string taking into
    account the user time zone

    Keep in mind that this conversion might change the actual day if the
    habit was entered 'early' or 'late' in the day.  This is correct because
    the user entered the habit in their own timezone, but the app stores this
    internally (and exports) in utc.  So, here we are effectively converting
    the time back to when the user actually entered it, based on the timezone
    the user claims they were in.
    """

    # We know habit list stores in UTC so don't need the timezone info
    dt = dt.split('+')[0].strip()
    dtime_obj = datetime.strptime(dt, '%Y-%m-%d %H:%M:%S')

    # Tell native datetime object we are using UTC, then we need to convert
    # that UTC time into the user's timezone BEFORE stripping off the time
    # to make sure the year, month, and date take into account timezone
    # differences.

    utc = dtime_obj.replace(tzinfo=utc_time_zone)
    return utc.astimezone(user_time_zone)


def _habits_to_markdown(habits):
    """Create markdown list of habits"""

    # FIXME: This is inefficient but not sure of a good way to use join since
    # we want to add a chacter to the beginning and end of each string in list.
    markdown = ''

    for habit, dt_obj in habits:
        markdown += '- [%02d:%02d] %s\n' % (dt_obj.hour, dt_obj.minute, habit)

    return markdown



def create_habitlist_entry(directory, day_str, habits, verbose):
    """Create day one file entry for given habits, date pair"""

    # Create unique uuid without any specific machine information
    # (uuid() vs.  uuid()) and strip any '-' characters to be
    # consistent with dayone format.
    uuid_str = re.sub('-', '', str(uuid.uuid4()))

    file_name = '%s.doentry' % (uuid_str)
    full_file_name = os.path.join(directory, file_name)

    date = convert_to_dayone_date_string(day_str)
    habits = _habits_to_markdown(habits)

    entry = {'entry_title': HEADER_FOR_DAYONE_ENTRIES,
              'habits': habits,'date': date, 'uuid_str': uuid_str}

    with open(full_file_name, 'w') as file_obj:
        text = ENTRY_TEMPLATE.format(**entry)
        file_obj.write(text)

    if verbose:
        print 'Created entry for %s: %s' % (date, file_name)


def parse_habits_file(filename, start_date=None):
    """
    Parse habits json file and return dict of data organized by day

    start_date can be a datetime object used only to return habits that were
    started on or after start_date
    """

    with open(filename, 'r') as file_obj:
        # FIXME: This expects 3 lines of junk at the beginning of the file, but
        # we could just read until we find '[' and ignore up until that point.
        junk = file_obj.readline()
        junk = file_obj.readline()
        junk = file_obj.readline()

        # FIXME: For my sample this is about 27kb of memory
        _json = file_obj.read()

    # FIXME: Downside here is that we assume the user was in the same timezone
    # for every habit.  However, it's possible that some of the habits were
    # entered while the user was traveling in a different timezone, etc.
    iphone_time_zone = _user_time_zone()
    utc_time_zone = tz.gettz('UTC')

    # Use a set b/c we can only do each habit once a day
    habits = collections.defaultdict(set)

    # FIXME: Maybe optimize this to not hold it all in memory

    # We have to parse all json and return it b/c the data is organized by
    # habit and we need it organized by date. So, we can't use a generator or
    # anything to yield values as they come b/c we won't know if we've parsed
    # the entire day until all JSON is parsed.

    # FIXME: Should have something to catch ValueError exceptions around this
    # so we can show the line with the error if something is wrong.
    for habit in json.loads(_json):
        name = habit['name']

        for dt in habit['completed']:
            dt_obj = _user_time_zone_date(dt, iphone_time_zone, utc_time_zone)
            if start_date is None or dt_obj >= start_date:
                # Habits will be organized by day then each one will have it's
                # own time.
                day_str = dt_obj.strftime('%Y-%m-%d')
                habits[day_str].add((name, dt_obj))

    return habits


def main():
    args = _parse_args()

    if args['test']:
        directory = './test'
        try:
            os.mkdir(directory)
        except OSError as err:
            print 'Warning: %s' % (err)
    else:
        directory = DAYONE_ENTRIES

    habits = parse_habits_file(args['input_file'], args['since'])

    for day_str, days_habits in habits.iteritems():
        create_habitlist_entry(directory, day_str, days_habits, args['verbose'])


if __name__ == '__main__':
    main()



"""
Simple script to parse idonethis.com CSV exported file and turn into individual
day one entries

To use this script you must first export your idonethis.com data in CSV format.
This can be done by performing the following:

1. Login to idonethis.com account
2. Go to settings (Gear icon in top-right navigation)
3. Choose 'Export' from menu
4. Click download button

This will save the CSV file which you can use directly as an argument to this
script.

You then will want to edit DAYONE_ENTRIES and possibly
HEADER_FOR_DAYONE_ENTRIES to match your data file and preferences.

There is also a '-t' option that will process your CSV file and put the results
in a 'test/' directory for inspection.  This is recommended before running this
application on your full Day One directory.

You can find all your imported idonethis entries after pushing the Day One by
searching in Day One for the #idonethis tag.  All imported entries will have
this tag.
"""

import argparse
import csv
from datetime import datetime
import os
import re
import uuid

from dayonetools.services import convert_to_dayone_date_string

DAYONE_ENTRIES = '/Users/durden/Dropbox/Apps/Day One/Journal.dayone/entries/'

# Depending on where you entered your iDoneThis entry the text might be wrapped
# in quotes.  This will automatically remove them.
STRIP_QUOTES = True

# This text will be inserted into the first line of all entries created, set to
# '' to remove this completely.
HEADER_FOR_DAYONE_ENTRIES = 'iDoneThis entry'


# Note the strange lack of indentation on the {entry_text} b/c day one will
# display special formatting to text that is indented, which we want to avoid.
ENTRY_TEMPLATE = """
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
<dict>
    <key>Creation Date</key>
    <date>{date}</date>
    <key>Entry Text</key>
    <string> {entry_title}
<![CDATA[
{entry_text}]]>
#idonethis
    </string>
    <key>Starred</key>
    <false/>
    <key>Tags</key>
    <array>
        <string>idonethis</string>
    </array>
    <key>UUID</key>
    <string>{uuid_str}</string>
</dict>
</plist>
"""


def _parse_args():
    """Parse sys.argv arguments"""

    parser = argparse.ArgumentParser(
                                description='Export iDonethis data to Day One')

    parser.add_argument('-f', '--file', action='store',
                        dest='input_file', required=True,
                        help='CSV file to import from')

    parser.add_argument('-v', '--verbose', default=False, action='store_true',
                        dest='verbose', required=False,
                        help='Verbose debugging information')

    def _datetime(str_):
        """Convert date string in YYYY-MM-DD format to datetime object"""

        if not str_:
            return None

        try:
            date = datetime.strptime(str_, '%Y-%m-%d')
        except ValueError:
            msg = 'Invalid date format, should be YYYY-MM-DD'
            raise argparse.ArgumentTypeError(msg)

        return date

    parser.add_argument('-s', '--since', type=_datetime,
                        help=('Only process entries starting with YYYY-MM-DD '
                              'and newer'))
    parser.add_argument('-t', '--test', default=False, action='store_true',
                        dest='test', required=False,
                        help=('Test import by creating Day one files in local '
                             'directory for inspect'))

    return vars(parser.parse_args())


def _create_dayone_entry(date, entries, directory, verbose):
    """Create single dayone journal entry for list of given entries"""

    entry_text = '\n'.join(entries)
    date = convert_to_dayone_date_string(date)

    # Create unique uuid without any specific machine information (uuid() vs.
    # uuid()) and strip any '-' characters to be consistent with dayone format.
    uuid_str = re.sub('-', '', str(uuid.uuid4()))

    file_name = '%s.doentry' % (uuid_str)
    full_file_name = os.path.join(directory, file_name)

    with open(full_file_name, 'w') as file_obj:
        text = ENTRY_TEMPLATE.format(entry_title=HEADER_FOR_DAYONE_ENTRIES,
                                     date=date,
                                     entry_text=entry_text,
                                     uuid_str=uuid_str)
        file_obj.write(text)

    if verbose:
        print 'Created entry for %s: %s' % (date, file_name)


def _sanitize_entry_text(entry_lines, strip_quotes):
    """
    Sanitize entry text by remove extra whitespace and quotes if asked
    """

    # Allow commas in the entry, everything after date is entry
    # Strip any line endings, we'll standardize them later
    entry_text = ''.join(entry_lines).strip()
    if strip_quotes:
        entry_text = re.sub('"', '', entry_text)

    return entry_text


def read_entries_by_day(filename, start_date=None):
    """
    Parse given CSV file of idonethis entries and yield tuple containting the
    following:
        (date, list of entries for day)

    date will be string in format: YYYY-MM-DD

    if start_date is provided as a datetime object then only days starting on
    or after the start_date will be returned.
    """

    with open(filename, 'r') as file_obj:
        current_day_entries = []
        curr_date = None
        date_re = re.compile('^\d{4}-\d{2}-\d{2}$')

        for row in csv.reader(file_obj):
            # If line doesn't start with a date assume it's just another line
            # in the current entry we are accumulating.
            new_date = row[0]
            if not date_re.match(new_date):
                entry_text = _sanitize_entry_text(row[1:], STRIP_QUOTES)

                if not current_day_entries:
                    raise IndexError(
                        'No current entries to add to, possibly invalid file')

                current_day_entries.append(entry_text)
                continue

            entry_text = _sanitize_entry_text(row[1:], STRIP_QUOTES)

            if curr_date is None:
                curr_date = new_date

            # iDoneThis orders file export in decreasing dates so when we find
            # an entry that occured BEFORE date user asked for we are done.
            if start_date is not None:
                curr_dtime_obj = datetime.strptime(curr_date, '%Y-%m-%d')
                if curr_dtime_obj < start_date:
                    raise StopIteration

            if curr_date == new_date:
                current_day_entries.append(entry_text)
            else:
                yield (curr_date, current_day_entries)
                current_day_entries = [entry_text]
                curr_date = new_date

        # Get last day if there was anything left over
        if current_day_entries:
            yield (curr_date, current_day_entries)


def main():
    args = _parse_args()

    if args['test']:
        directory = './test'
        try:
            os.mkdir(directory)
        except OSError as err:
            print 'Warning: %s' % (err)
    else:
        directory = DAYONE_ENTRIES

    for curr_date, entries in read_entries_by_day(args['input_file'],
                                                  args['since']):
        entries = reversed(entries)

        _create_dayone_entry(curr_date, entries, directory, args['verbose'])


if __name__ == '__main__':
    main()



# -*- mode: python;  coding: utf-8; -*-

"""Common services code"""

import os

AVAILABLE_SERVICES = ['habit_list', 'idonethis', 'nikeplus', 'pedometerpp']


def get_service_module(service_name):
    """Import given service from dayonetools.services package"""

    import importlib
    services_pkg = 'dayonetools.services'

    module = '%s.%s' % (services_pkg, service_name)
    return importlib.import_module(module)


def convert_to_dayone_date_string(day_str):
    """
    Convert given date in 'yyyy-mm-dd' format into dayone accepted format of
    iso8601

    The timestamp will match midnight but year, month, and day will be replaced
    with given arguments.
    """

    year, month, day = day_str.split('-')

    from datetime import datetime
    now = datetime.utcnow()

    # FIXME: The current version of day one does not support timezone data
    # correctly.  So, if we enter midnight here then every entry is off by a
    # day.

    # Don't know the hour, minute, etc. so just assume midnight
    date = now.replace(year=int(year),
                       month=int(month),
                       day=int(day),
                       minute=00,
                       hour=10,
                       second=0,
                       microsecond=0)

    iso_string = date.isoformat()

    # Very specific format for dayone, if the 'Z' is not in the
    # correct positions the entries will not show up in dayone at all.
    return iso_string + 'Z'


def get_outfolder_names(service, outfolderseed, verbose=False):
    """
    Ensure existence of a temp folder and return the path
    Assemble path to the outfolder according to the given seed
    Search for the Day One Journal in iCloud, DropBox folders and return the path
    or return the temp folder if no journal was found
    :param service: Name of the calling service
    :param outfolderseed: command line seed for the outfolder
    :param verbose: wie der Name schon sagt
    """

    # FIXME: Add progress output for --verbose

    tempfolder = os.path.abspath(os.path.join('..', 'temp', service))
    if not os.path.exists(tempfolder):
        os.makedirs(tempfolder)

    d1folder = ''
    if outfolderseed == 'test':
        # force it into the tempfolder
        d1folder = tempfolder
    elif outfolderseed == 'auto':
        # Try to find the Day One journal at the usual suspect places
        # FIXME: What is the correct name, I’m not using DropBox
        #candidate = os.path.expanduser('~/Dropbox/Apps/Day One/Journal.dayone/entries')
        candidate = os.path.expanduser('~/Dropbox/Applications/Day One/Journal.dayone/entries')
        if os.path.exists(candidate):
            d1folder = candidate
        else:
            candidate = os.path.expanduser(
                '~/Library/Mobile Documents/5U8NS4GX82~com~dayoneapp~dayone/Documents/Journal_dayone/entries'
            )
            if os.path.exists(candidate):
                d1folder = candidate
    elif os.path.exists(outfolderseed):
        # We try to use the given folder
        d1folder = outfolderseed

    if d1folder == '':
        # TODO: issue a warning or flag an error
        d1folder = tempfolder

    return d1folder, tempfolder


# Make all services available from this level
for service_name in AVAILABLE_SERVICES:
    service = get_service_module(service_name)



from setuptools import setup
import io

# Take from Jeff Knupp's excellent article:
# http://www.jeffknupp.com/blog/2013/08/16/open-sourcing-a-python-project-the-right-way/

def read(*filenames, **kwargs):
    encoding = kwargs.get('encoding', 'utf-8')
    sep = kwargs.get('sep', '\n')
    buf = []

    for filename in filenames:
        with io.open(filename, encoding=encoding) as f:
            buf.append(f.read())

    return sep.join(buf)


setup(name='dayonetools',
      version='1.1.0',
      description='Tools to import multiple services into Day One Journal',
      long_description=read('README.md'),
      package_data={'': ['README.md']},
      license='MIT',
      author='Luke Lee',
      author_email='durdenmisc@gmail.com',
      url='https://github.com/durden/dayonetools',
      packages=['dayonetools', 'dayonetools.services'],
      install_requires=['python-dateutil>=2.2', 'pytz==2014.4'],
      platforms='any',
      classifiers= [
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Topic :: System :: Archiving'],
      entry_points={
        "console_scripts": [
            "dayonetools = dayonetools.main:main",
        ]
    },
)



"""Entry point for dayonetools import process"""

import sys

import dayonetools.services as services


def _show_help():
    """Print help"""

    _help = "{doc}\n\nUsage: {usage}\nSupported service arguments: {services}"
    args = {'doc': __doc__,
            'usage': '[--version] [%s <service_name>]' % (sys.argv[0]),
            'services': services.AVAILABLE_SERVICES}

    print _help.format(**args)


def _parse_args():
    """
    Parse sys.argv arguments

    Don't use argparse here b/c we are just a front-end to other scripts that
    use argparse.  This script only has one real argument, a service, and we
    forward all other args down to main scripts.  Argparse is a pain when
    wanting to forward arguments to other scripts esp. if the lower ones take
    arguments this one doesn't 'accept.'
    """

    args = {}

    try:
        args['service_name'] = sys.argv[1]
    except IndexError:
        _show_help()
        sys.exit(-1)

    if args['service_name'] == '--version':
        from dayonetools import __version__
        print __version__
        sys.exit(0)

    valid_service = args['service_name'] in services.AVAILABLE_SERVICES

    if valid_service:
        args['service_module'] = services.get_service_module(
                                                        args['service_name'])

    # Manually handle -h so we can pass it to other scripts instead of this
    # main entry point stealing it.
    if '-h' in sys.argv and valid_service:
        pass
    elif '-h' in sys.argv or len(sys.argv) == 2 or not valid_service:
        _show_help()
        sys.exit(0)

    return args


def main():
    args = _parse_args() 

    # We steal the first two arguments, service name and script.
    sys.argv[0] = args['service_name']
    sys.argv.remove(args['service_name'])

    args['service_module'].main()


if __name__ == '__main__':
    main()
